#include "StdAfx.h"
#include "Comm.h"
#include "GGame.h"
#include "GStrings.h"


/*
GGame 
*/
GGame::GGame() : m_pd3d(NULL), m_pd3dd(NULL), m_pspr(NULL), 
	m_pline(NULL), m_activated(true),
	m_pstate(NULL), m_pnewstate(NULL), m_drawskip(false)
{
	m_dev.d3ddt=D3DDEVTYPE_HAL;
	m_dev.buffercount=0;
	m_dev.fullscreen=0;
	m_dev.backformat=D3DFMT_X8R8G8B8;
	m_dev.d3dse=D3DSWAPEFFECT_DISCARD;
	m_dev._size=0;
	m_dev.width=0;
	m_dev.height=0;
	m_dev.stencilformat=D3DFMT_UNKNOWN;
	m_dev.refreshrate=0;
}

/*
GGame Ҹ
*/
GGame::~GGame()
{
	ReleaseState();
	Release3D();
}

GRET GGame::ReleaseState()
{
	SAFE_DELETE(m_pnewstate);
	SAFE_DELETE(m_pstate);
	return GRET_OK;
}

/*
GGame::Release3D()

Ǿ D3Dġ Ѵ.
*/
GRET GGame::Release3D()
{
	SAFE_RELEASE(m_pline);
	SAFE_RELEASE(m_pspr);
	SAFE_RELEASE(m_pd3dd);
	SAFE_RELEASE(m_pd3d);
	return 0;
}

/*
GGame::_getdpp()

m_dev  ġ Ӽ
D3D 밡 D3DPRESENT_PARAMETERS Ѵ.
*/
D3DPRESENT_PARAMETERS GGame::_getdpp()
{
	D3DCAPS9 d3dc;
	m_pd3d->GetDeviceCaps(D3DADAPTER_DEFAULT, m_dev.d3ddt, &d3dc);
	D3DPRESENT_PARAMETERS d3dpp;
	memset(&d3dpp, 0, sizeof(d3dpp));
	if(m_dev.fullscreen)
	{
		d3dpp.BackBufferWidth=m_dev.width;
		d3dpp.BackBufferHeight=m_dev.height;
		if(m_dev.backformat==D3DFMT_UNKNOWN)m_dev.backformat=D3DFMT_X8R8G8B8;
		d3dpp.FullScreen_RefreshRateInHz=m_dev.refreshrate;
	}
	else
	{
		d3dpp.BackBufferWidth=0;
		d3dpp.BackBufferHeight=0;
		d3dpp.FullScreen_RefreshRateInHz=0;
	}
	d3dpp.Windowed=!m_dev.fullscreen;
	d3dpp.BackBufferCount=m_dev.buffercount;
	d3dpp.BackBufferFormat=m_dev.backformat;
	d3dpp.SwapEffect=m_dev.d3dse;
	if(m_dev.stencilformat!=0)
	{
		d3dpp.AutoDepthStencilFormat=m_dev.stencilformat;
		d3dpp.EnableAutoDepthStencil=true;
	}
	return d3dpp;
}

/*
GGame::Create3DDevice(HWND hWnd)
hWnd ġ   ڵ̴.

IDirect3D9  Ŀ ̸ ̿
IDirect3DDevice9 .
׸ ID3DXSprite ID3DXLine .
*/
GRET GGame::Create3DDevice(HWND hWnd)
{
	m_pd3d=Direct3DCreate9(D3D_SDK_VERSION);
	if(m_pd3d==NULL)return GRET_ERROR_NOD3D;
	D3DPRESENT_PARAMETERS d3dpp=_getdpp();
	if( FAILED(m_pd3d->CreateDevice(D3DADAPTER_DEFAULT, m_dev.d3ddt, hWnd,
		D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &m_pd3dd)) )return GRET_ERROR_FAIL;
	InitRenderState();
	if(FAILED( D3DXCreateSprite(m_pd3dd, &m_pspr) ))return GRET_ERROR_FAIL;
	if(FAILED( D3DXCreateLine(m_pd3dd, &m_pline) ))return GRET_ERROR_FAIL;
	m_dev.hwnd=hWnd;
	return 0;
}

/*
GGame::SetRenderState()

RenderState ʱȭѴ.
*/
void GGame::InitRenderState()
{
	/*ID3DXSprite::Begin D3DXSPRITE_DONOTMODIFY_RENDERSTATE ɼ ༭ ȣ ̹Ƿ
	ID3DXSprite ڵ ִ RenderState
	   ־ Spriter  µȴ.*/
	m_pd3dd->SetRenderState(D3DRS_ALPHABLENDENABLE, true);
	m_pd3dd->SetRenderState(D3DRS_ALPHATESTENABLE, true);
	m_pd3dd->SetRenderState(D3DRS_ALPHAFUNC, D3DCMP_GREATER);
	m_pd3dd->SetRenderState(D3DRS_ALPHAREF, 0);
	m_pd3dd->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
	m_pd3dd->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
	m_pd3dd->SetRenderState(D3DRS_BLENDOP, D3DBLENDOP_ADD);
	m_pd3dd->SetRenderState(D3DRS_ZENABLE, false);
	m_pd3dd->SetRenderState(D3DRS_LIGHTING, false);
	m_pd3dd->SetRenderState(D3DRS_CULLMODE, D3DCULL_NONE);
	m_pd3dd->SetTextureStageState(0, D3DTSS_COLORARG1, D3DTA_TEXTURE);
	m_pd3dd->SetTextureStageState(0, D3DTSS_COLORARG2, D3DTA_DIFFUSE);
	m_pd3dd->SetTextureStageState(0, D3DTSS_COLOROP, D3DTOP_MODULATE);
	m_pd3dd->SetTextureStageState(0, D3DTSS_ALPHAARG1, D3DTA_TEXTURE);
	m_pd3dd->SetTextureStageState(0, D3DTSS_ALPHAARG2, D3DTA_DIFFUSE);
	m_pd3dd->SetTextureStageState(0, D3DTSS_ALPHAOP, D3DTOP_MODULATE);
	m_pd3dd->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
	m_pd3dd->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	m_pd3dd->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
	/*ٽ ۸ ϱ  غ*/
	m_pd3dd->SetRenderState(D3DRS_STENCILENABLE, true);
	m_pd3dd->SetRenderState(D3DRS_STENCILFUNC, D3DCMP_ALWAYS);
	m_pd3dd->SetRenderState(D3DRS_STENCILREF, 1);
}

/*
GGame::Reset3DDevice()

3Dġ ٽ Ѵ.
*/
GRET GGame::Reset3DDevice()
{
	if(m_pd3d==NULL)return GRET_ERROR_NOD3D;
	D3DPRESENT_PARAMETERS d3dpp=_getdpp();
	if( FAILED(m_pd3dd->Reset(&d3dpp)) )return GRET_ERROR_FAIL;
	InitRenderState();
	return 0;
}

/*
GGame::SetState(MState* newstate)

 ¸ ϰ, newstate ο · Ѵ.
*/
GRET GGame::SetState(MState* newstate)
{
	if(m_pnewstate)delete m_pnewstate;
	m_pnewstate=newstate;
	m_stateinit=false;
	return 0;
}

/*
GGame::MessageLoop()

    ޼ ޾Ƶ̰,
 ó Ѵ.
޼  쿡 ӷ Ѵ.
*/
GRET GGame::MessageLoop()
{
	if(!m_pd3dd)return GRET_ERROR_NODEVICE;
	bool islost=false;
	MSG msg;
	OnCreate();
	while(1)
	{
		if(PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{
			if(msg.message==WM_QUIT)return 0;
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
		else
		{
			if(m_pnewstate)
			{
				if(m_pstate)delete m_pstate;
				m_pstate=m_pnewstate;
				m_pnewstate=NULL;
			}
			if(!m_stateinit)//MState OnCreate ȣ ʾҴٸ
			{
				if(m_pstate)m_pstate->OnCreate();//ȣش.
				m_stateinit=true;
			}
			OnProc();

			if(islost)
			{
				if((!m_dev.fullscreen || m_activated) && Reset3DDevice()==GRET_OK)
				{
					islost=false;
					OnReset();
				}
			}
			else if(m_drawskip)
			{
				OnDrawSkip();
			}
			else
			{
				HRESULT hr=m_pd3dd->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_STENCIL, D3DCOLOR_XRGB(64,128,192), 0.f, 0);
				if( SUCCEEDED(m_pd3dd->BeginScene()) )
				{
					m_pspr->Begin(D3DXSPRITE_DONOTMODIFY_RENDERSTATE | D3DXSPRITE_DONOTSAVESTATE);
					/*D3DXSPRITE_DONOTMODIFY_RENDERSTATE
					 : Begin  RenderState ٲ ʰؼ ʿ RenderState δ.*/
					OnDraw();
					m_pspr->End();
					m_pd3dd->EndScene();
					if(m_pd3dd->Present(NULL, NULL, NULL, NULL)==D3DERR_DEVICELOST)
					{
						islost=true;
						OnLost();
					}
				}
			}
			OnPostProc();
			m_drawskip=false;
		}
	}
	return 0;
}

/*
GGame::MsgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)

ݹԼ WndProc ,  ޼ óѴ.
*/
LRESULT GGame::MsgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
	case WM_ACTIVATEAPP:
		m_activated=(wParam!=0);
		return 0;
	default:
		return DefWindowProc(hWnd, uMsg, wParam, lParam);
	}
}



/*
GGame::DrawLine(int x1, int y1, int x2, int y2, D3DCOLOR color)

ȭ鿡 (x1, y1) (x2, y2) ̸  color ׸.
*/
GRET GGame::DrawLine(int x1, int y1, int x2, int y2, D3DCOLOR color)
{
	D3DXVECTOR2 v[2];
	v[0].x=(float)x1;
	v[0].y=(float)y1;
	v[1].x=(float)x2;
	v[1].y=(float)y2;
	m_pspr->Flush();
	if(FAILED( m_pline->Draw(v, 2, color) ))
	{
		return GRET_ERROR_FAIL;
	}
	return 0;
}

/*
GGame::SetLineWidth(int w)

 ⸦ w Ѵ.
*/
GRET GGame::SetLineWidth(int w)
{
	if(FAILED( m_pline->SetWidth((float)w) )) return GRET_ERROR_FAIL;
	return 0;
}

/*
GGame::DrawRectangle(int x, int y, int w, int h, D3DCOLOR color)

ȭ  (x, y) ΰ w, ΰ h 簢 color ĥѴ.
*/
GRET GGame::DrawRectangle(int x, int y, int w, int h, D3DCOLOR color)
{
	if(!m_pd3dd)return GRET_ERROR_NODEVICE;
	m_pspr->Flush();
	m_pd3dd->SetFVF(D3DFVF_XYZRHW | D3DFVF_DIFFUSE);
	m_pd3dd->SetTexture(0, NULL);
	struct{
		float x, y, z, w;
		D3DCOLOR color;
	} vert[4]={
		x-0.5f, y-0.5f, 1.f, 1.f, color,
		x+w-0.5f, y-0.5f, 1.f, 1.f, color,
		x-0.5f, y+h-0.5f, 1.f, 1.f, color,
		x+w-0.5f, y+h-0.5f, 1.f, 1.f, color
	};
	m_pd3dd->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, vert, 20);
	return 0;
}

/*
GGame::DrawRectangle(int x, int y, int w, int h, 
	D3DCOLOR color1, D3DCOLOR color2, D3DCOLOR color3, D3DCOLOR color4)

ȭ  (x, y) ΰ w, ΰ h 簢 ׸.
  color1,   color2,  Ʒ color3,  Ʒ color4
 ׶̼ ĥѴ.
*/
GRET GGame::DrawRectangle(int x, int y, int w, int h, 
				D3DCOLOR color1, D3DCOLOR color2, D3DCOLOR color3, D3DCOLOR color4)
{
	if(!m_pd3dd)return GRET_ERROR_NODEVICE;
	m_pspr->Flush();
	m_pd3dd->SetFVF(D3DFVF_XYZRHW | D3DFVF_DIFFUSE);
	m_pd3dd->SetTexture(0, NULL);
	struct{
		float x, y, z, w;
		D3DCOLOR color;
	} vert[4]={
		x-0.5f, y-0.5f, 1.f, 1.f, color1,
		x+w-0.5f, y-0.5f, 1.f, 1.f, color2,
		x-0.5f, y+h-0.5f, 1.f, 1.f, color3,
		x+w-0.5f, y+h-0.5f, 1.f, 1.f, color4
	};
	m_pd3dd->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, vert, 20);
	return 0;
}

/*
GGame::SetDrawMode(int mode, int cmode)

׸带 Ѵ.
*/

GRET GGame::SetDrawMode(int mode, int cmode)
{
	if(!m_pd3dd)return GRET_ERROR_NODEVICE;
	m_pspr->Flush(); /*¸   ׵ ׷ȴ ͵
					 Flush()Լ   Ѵ.*/
	m_pd3dd->SetRenderState(D3DRS_COLORWRITEENABLE, cmode);
	switch(mode)
	{
	case DM_Normal: // Result = Src*SrcAlpha + Dest*(1-SrcAlpha)
		m_pd3dd->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
		m_pd3dd->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA);
		m_pd3dd->SetRenderState(D3DRS_BLENDOP, D3DBLENDOP_ADD);
		break;
	case DM_NoBlend: // Result = Src*1 + Dest*0
		m_pd3dd->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_ONE);
		m_pd3dd->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);
		m_pd3dd->SetRenderState(D3DRS_BLENDOP, D3DBLENDOP_ADD);
		break;
	case DM_Add: // Result = Src*SrcAlpha + Dest*1
		m_pd3dd->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
		m_pd3dd->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
		m_pd3dd->SetRenderState(D3DRS_BLENDOP, D3DBLENDOP_ADD);
		break;
	case DM_Subtract: // Result = Dest*1 - Src*SrcAlpha
		m_pd3dd->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_SRCALPHA);
		m_pd3dd->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ONE);
		m_pd3dd->SetRenderState(D3DRS_BLENDOP, D3DBLENDOP_REVSUBTRACT);
		break;
	case DM_Multiply: // Result = Src*DestColor + Dest*0
		m_pd3dd->SetRenderState(D3DRS_SRCBLEND, D3DBLEND_DESTCOLOR);
		m_pd3dd->SetRenderState(D3DRS_DESTBLEND, D3DBLEND_ZERO);
		m_pd3dd->SetRenderState(D3DRS_BLENDOP, D3DBLENDOP_ADD);
		break;
	}
	return 0;
}

/*
GGame::SetDrawSkip(bool skip)

 ӿ ׸⸦   Ѵ.
*/
GRET GGame::SetDrawSkip(bool skip)
{
	m_drawskip=skip;
	return 0;
}

/*
GGame::GetDrawSkip()

 ӿ ׸Ⱑ   Ѵ.
*/
bool GGame::GetDrawSkip()
{
	return m_drawskip;
}

/*
GGame::GetInstance()

 GGame νϽ ȯѴ.
*/
GGame* GGame::GetInstance()
{
	static GGame inst;
	return &inst;
}

/*
GGame::GetDevice()

IDirect3DDevice9 ġ ȯѴ.
*/
LPDIRECT3DDEVICE9 GGame::GetDevice()
{
	return m_pd3dd;
}

/*
GGame::GetSprite()

ID3DXSprite ġ ȯѴ.
*/
LPD3DXSPRITE GGame::GetSprite()
{
	return m_pspr;
}

/*
GGame::OnCreate()

GGame νϽ ޼ 
ȣȴ.
*/
void GGame::OnCreate()
{
	m_time=timeGetTime();
}

/*
GGame::OnProc()

GGame   óҶ
ȣȴ.
*/
void GGame::OnProc()
{
	DWORD t=timeGetTime();
	if(m_pstate)m_pstate->OnProc(t-m_time);
	m_time=t;
}

/*
GGame::OnDraw()

GGame ׸⸦ Ҷ ȣȴ.
ӷƮ   
ȣ   ִ.
*/
void GGame::OnDraw()
{
	if(m_pstate)m_pstate->OnDraw();
}

/*
GGame::OnPostProc()

GGame ׸ó (Ȥ ϰ)
ó    ȣȴ.
*/
void GGame::OnPostProc()
{
	if(m_pstate)m_pstate->OnPostProc();
}

/*
GGame::OnLost()

GGame D3Dġ Ҿ  ȣȴ.
*/
void GGame::OnLost()
{
	m_pspr->OnLostDevice();
	m_pline->OnLostDevice();
	if(m_pstate)m_pstate->OnLost();
}

/*
GGame::OnReset()

GGame D3Dġ   ȣȴ.
*/
void GGame::OnReset()
{
	m_pspr->OnResetDevice();
	m_pline->OnResetDevice();
	if(m_pstate)m_pstate->OnReset();
}

/*
GGame::OnDrawSkip()

 ӿ ׸Ⱑ Ǿ 
OnDrawſ  Լ ȣȴ.
*/
void GGame::OnDrawSkip()
{
	if(m_pstate)m_pstate->OnDrawSkip();
}
